local ShowMyDPS = LibStub("AceAddon-3.0"):NewAddon("ShowMyDPS", "AceConsole-3.0", "AceEvent-3.0")
local modes = {"DPS", "HPS"}--, "DTPS", "HTPS"}

local DBDefault = {
	char = {
		mode = 1,
		width = 220,
		height = 25,
		isLock = false,
		isShown = true,
		oocHide = false,
		customDisplay = "$total | $persec | $time",
		font = "Fonts\\FRIZQT__.TTF",
		fontSize = 10,
	}
}

function ShowMyDPS:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("SMDdb", DBDefault)
	self:RegisterChatCommand("smd","SlashHandler")
end

function ShowMyDPS:OnEnable()
	self:CreateFrame()
	self:RegisterEvent("PLAYER_REGEN_ENABLED","LeaveCombat")
	self:RegisterEvent("PLAYER_REGEN_DISABLED","EnterCombat")
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED","AddAmount")
end

function ShowMyDPS:CreateFrame()
	local smdFrame = CreateFrame("Frame", "ShowMyDPSFrame", UIParent, "GameTooltipTemplate")
	smdFrame:SetPoint("CENTER", UIParent, self.db.char.x and "BOTTOMLEFT" or "CENTER", self.db.char.x or 0, self.db.char.y or 0)
	
	smdFrame:EnableMouse(true)
	smdFrame:SetToplevel(true)
	smdFrame:SetMovable(true)
	smdFrame:SetFrameStrata("LOW")
	smdFrame:SetWidth(self.db.char.width)
	smdFrame:SetHeight(self.db.char.height)
	if self.db.char.isShown and not self.db.char.oocHide then
		smdFrame:Show()
	end
	self.db.char.x, self.db.char.y = smdFrame:GetCenter()
	local backdrop_header = {bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
			edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile=1, tileSize=16, edgeSize = 16,
			insets = {left = 5, right = 5, top = 5, bottom = 5}}

	smdFrame:SetBackdrop(backdrop_header);
	smdFrame:SetBackdropBorderColor(TOOLTIP_DEFAULT_COLOR.r, TOOLTIP_DEFAULT_COLOR.g, TOOLTIP_DEFAULT_COLOR.b)
	smdFrame:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)

	smdFrame.text = smdFrame:CreateFontString("$parentText", "ARTWORK", "GameFontNormalSmall")
	smdFrame.text:SetFont(self.db.char.font, self.db.char.fontSize)
	smdFrame.text:SetPoint("CENTER", smdFrame, "CENTER", 0, 0)
	smdFrame.text:Show()

	smdFrame:SetScript("OnMouseDown", function(frame, button) 
		if button == "LeftButton" and not self.db.char.isLock then
			frame.isMoving = true
			frame:StartMoving()	
		end
	end)

	smdFrame:SetScript("OnMouseUp", function(frame, button) 
		if button == "LeftButton" and not self.db.char.isLock then
			if( frame.isMoving ) then
				frame.isMoving = nil
				frame:StopMovingOrSizing()
				if self.db.char.x ~= frame:GetCenter() and self.db.char.y ~= select(2, frame:GetCenter()) then
					self.db.char.x, self.db.char.y = frame:GetCenter()
				else
					self.db.char.mode = self.db.char.mode + 1
					if self.db.char.mode > #modes then self.db.char.mode = 1 end
					self:RefreshFrame()
				end
			end
		elseif button == "LeftButton" then
			self.db.char.mode = self.db.char.mode + 1
			if self.db.char.mode > #modes then self.db.char.mode = 1 end
			self:RefreshFrame()
		else
			self:ToggleOptionsFrame()
		end
	end)
	
	self:RefreshFrame()
	self.Frame = smdFrame
	-- self:CreateOptionsFrame()
end

function ShowMyDPS:ToggleOptionsFrame()
	if self.OptionsFrame and not self.OptionsFrame:IsShown() then
		self.OptionsFrame:Show()
	elseif self.OptionsFrame then
		self.OptionsFrame:Hide()
	else
		self:CreateOptionsFrame()
	end
end

function ShowMyDPS:CreateOptionsFrame()
	local smdOFrame = CreateFrame("Frame", "ShowMyDPSOPtionsFrame", UIParent, "GameTooltipTemplate")
	smdOFrame:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	smdOFrame:EnableMouse(true)
	smdOFrame:SetToplevel(true)
	smdOFrame:SetFrameStrata("LOW")
	smdOFrame:SetWidth(220)
	smdOFrame:SetHeight(220)

	local backdrop_header = {bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
			edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile=1, tileSize=16, edgeSize = 16,
			insets = {left = 5, right = 5, top = 5, bottom = 5}}

	smdOFrame:SetBackdrop(backdrop_header);
	smdOFrame:SetBackdropBorderColor(TOOLTIP_DEFAULT_COLOR.r, TOOLTIP_DEFAULT_COLOR.g, TOOLTIP_DEFAULT_COLOR.b)
	smdOFrame:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)

	smdOFrame.title = smdOFrame:CreateFontString("$parentText", "ARTWORK", "GameFontNormalSmall")
	smdOFrame.title:SetPoint("TOP", smdOFrame, "TOP",0,-10)
	smdOFrame.title:SetText("Show my DPS : Options")
	smdOFrame.title:Show()
	
	smdOFrame.showBox = CreateFrame("CheckButton", "showBox", smdOFrame, "UICheckButtonTemplate")
	smdOFrame.showBox:SetPoint("TOPLEFT",smdOFrame,"TOPLEFT",8,-20)
	smdOFrame.showBox:Show()
	smdOFrame.showBox:SetScript("OnClick", function(self,button,down)
		if ShowMyDPS.db.char.isShown then
			ShowMyDPS.db.char.isShown = false
			ShowMyDPS.Frame:Hide()
		elseif ShowMyDPS.db.char.oocHide and not InCombatLockdown() then
			ShowMyDPS.db.char.isShown = true
		else
			ShowMyDPS.db.char.isShown = true
			ShowMyDPS.Frame:Show()
		end
	end)
	if ShowMyDPS.db.char.isShown then smdOFrame.showBox:SetChecked(true) end
	
	smdOFrame.showText = smdOFrame:CreateFontString("$parentText", "ARTWORK", "GameFontNormalSmall")
	smdOFrame.showText:SetPoint("TOPLEFT", smdOFrame, "TOPLEFT",38,-31)
	smdOFrame.showText:SetText("Show SMD")
	smdOFrame.showText:Show()
	
	smdOFrame.lockBox = CreateFrame("CheckButton", "lockBox", smdOFrame, "UICheckButtonTemplate")
	smdOFrame.lockBox:SetPoint("TOPLEFT",smdOFrame,"TOPLEFT",8,-50)
	smdOFrame.lockBox:Show()
	smdOFrame.lockBox:SetScript("OnClick", function(self,button,down)
		ShowMyDPS.db.char.isLock = not ShowMyDPS.db.char.isLock
	end)
	if ShowMyDPS.db.char.isLock then smdOFrame.lockBox:SetChecked(true) end
	
	smdOFrame.lockText = smdOFrame:CreateFontString("$parentText", "ARTWORK", "GameFontNormalSmall")
	smdOFrame.lockText:SetPoint("TOPLEFT", smdOFrame, "TOPLEFT",38,-61)
	smdOFrame.lockText:SetText("Lock frame")
	smdOFrame.lockText:Show()
	
	smdOFrame.oocHideBox = CreateFrame("CheckButton", "oocHideBox", smdOFrame, "UICheckButtonTemplate")
	smdOFrame.oocHideBox:SetPoint("TOPLEFT",smdOFrame,"TOPLEFT",8,-80)
	smdOFrame.oocHideBox:Show()
	smdOFrame.oocHideBox:SetScript("OnClick", function(self,button,down)
		ShowMyDPS.db.char.oocHide = not ShowMyDPS.db.char.oocHide
		if not InCombatLockdown() then 
			if ShowMyDPS.db.char.oocHide then ShowMyDPS.Frame:Hide() elseif ShowMyDPS.db.char.isShown then ShowMyDPS.Frame:Show() end
		end
	end)
	if ShowMyDPS.db.char.oocHide then smdOFrame.oocHideBox:SetChecked(true) end
	
	smdOFrame.oocHideText = smdOFrame:CreateFontString("$parentText", "ARTWORK", "GameFontNormalSmall")
	smdOFrame.oocHideText:SetPoint("TOPLEFT", smdOFrame, "TOPLEFT",38,-91)
	smdOFrame.oocHideText:SetText("Hide out of combat")
	smdOFrame.oocHideText:Show()
	
	smdOFrame.customDispEdit = CreateFrame("EditBox", "customDispEdit", smdOFrame, "InputBoxTemplate")
	smdOFrame.customDispEdit:SetPoint("TOPLEFT",smdOFrame,"TOPLEFT",12,-130)
	smdOFrame.customDispEdit:SetWidth(200)
	smdOFrame.customDispEdit:SetHeight(20)
	smdOFrame.customDispEdit:SetAutoFocus(false)
	smdOFrame.customDispEdit:SetText(ShowMyDPS.db.char.customDisplay)
	smdOFrame.customDispEdit:SetScript("OnTextChanged", function() 
		ShowMyDPS.db.char.customDisplay = smdOFrame.customDispEdit:GetText()
	end)
	smdOFrame.customDispEdit:Show()
	
	smdOFrame.customDispText = smdOFrame:CreateFontString("$parentText", "ARTWORK", "GameFontNormalSmall")
	smdOFrame.customDispText:SetPoint("TOPLEFT", smdOFrame, "TOPLEFT",13,-113)
	smdOFrame.customDispText:SetText("Custom display ($total/$persec/$time)")
	smdOFrame.customDispText:Show()
	
	smdOFrame.widthBox = CreateFrame("EditBox", "widthBox", smdOFrame, "InputBoxTemplate")
	smdOFrame.widthBox:SetPoint("TOPLEFT",smdOFrame,"TOPLEFT",65,-158)
	smdOFrame.widthBox:SetWidth(40)
	smdOFrame.widthBox:SetHeight(20)
	smdOFrame.widthBox:SetAutoFocus(false)
	smdOFrame.widthBox:SetText(floor(self.db.char.width))
	smdOFrame.widthBox:SetScript("OnChar", function() 
		ShowMyDPS.db.char.width = smdOFrame.widthBox:GetText()
		ShowMyDPS.Frame:SetWidth(ShowMyDPS.db.char.width)
	end)
	smdOFrame.widthBox:Show()
	
	smdOFrame.widthText = smdOFrame:CreateFontString("$parentText", "ARTWORK", "GameFontNormalSmall")
	smdOFrame.widthText:SetPoint("TOPLEFT", smdOFrame, "TOPLEFT",15,-163)
	smdOFrame.widthText:SetText("Width :")
	smdOFrame.widthText:Show()
	
	smdOFrame.heightBox = CreateFrame("EditBox", "heightBox", smdOFrame, "InputBoxTemplate")
	smdOFrame.heightBox:SetPoint("TOPLEFT",smdOFrame,"TOPLEFT",170,-158)
	smdOFrame.heightBox:SetWidth(40)
	smdOFrame.heightBox:SetHeight(20)
	smdOFrame.heightBox:SetAutoFocus(false)
	smdOFrame.heightBox:SetText(floor(ShowMyDPS.db.char.height))
	smdOFrame.heightBox:SetScript("OnChar", function() 
		ShowMyDPS.db.char.height = smdOFrame.heightBox:GetText()
		ShowMyDPS.Frame:SetHeight(ShowMyDPS.db.char.height)
	end)
	smdOFrame.heightBox:Show()
	
	smdOFrame.heightText = smdOFrame:CreateFontString("$parentText", "ARTWORK", "GameFontNormalSmall")
	smdOFrame.heightText:SetPoint("TOPLEFT", smdOFrame, "TOPLEFT",120,-163)
	smdOFrame.heightText:SetText("Height :")
	smdOFrame.heightText:Show()
	
	smdOFrame.okButton = CreateFrame("Button", "okButton", smdOFrame, "UIPanelButtonTemplate")
	smdOFrame.okButton:SetPoint("BOTTOMRIGHT",smdOFrame,"BOTTOMRIGHT",-5,5)
	smdOFrame.okButton:SetWidth(40)
	smdOFrame.okButton:SetHeight(20)
	smdOFrame.okButton:SetText("Ok")
	smdOFrame.okButton:SetScript("OnClick", function() 
		smdOFrame:Hide()
	end)
	smdOFrame.okButton:Show()
	
	smdOFrame.resetPos = CreateFrame("Button", "resetPosButton", smdOFrame, "UIPanelButtonTemplate")
	smdOFrame.resetPos:SetPoint("BOTTOMLEFT",smdOFrame,"BOTTOMLEFT",5,5)
	smdOFrame.resetPos:SetWidth(100)
	smdOFrame.resetPos:SetHeight(20)
	smdOFrame.resetPos:SetText("Reset position")
	smdOFrame.resetPos:SetScript("OnClick", function() 
		ShowMyDPS.Frame:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
		self.db.char.x, self.db.char.y = ShowMyDPS.Frame:GetCenter()
		smdOFrame.xPos:SetText(floor(ShowMyDPS.db.char.x))
		smdOFrame.yPos:SetText(floor(ShowMyDPS.db.char.y))
	end)
	smdOFrame.resetPos:Show()

	self.OptionsFrame = smdOFrame
	self.OptionsFrame:Show()
end

function ShowMyDPS:SlashHandler(msg)
	self:ToggleOptionsFrame()
end

local cbtDuration, cbtStart = nil, nil
local cbtDPS, cbtDmgAmount, cbtDmgShown = nil, nil, nil
local cbtHPS, cbtHealAmount, cbtHealShown = nil, nil, nil

function ShowMyDPS:EnterCombat()
	if self.db.char.isShown and self.db.char.oocHide then self.Frame:Show() end
	if cbtStart and cbtStart > 3.6 then
		cbtHealAmount = 0
		cbtDmgAmount = 0
		cbtStart = GetTime() - 3.5
	else
		self:RefreshFrame()
	end
end

function ShowMyDPS:AddAmount(Event, ...)
	local timestamp, event, hideCaster, sourceGUID, sourceName, sourceFlags, sourceFlags2, destGUID, destName, destFlags, destFlags2, meleeAmount, _, _, dmghealAmount, overHeal = ...
	
	if bit.band(sourceFlags, 0x1111) == 0x1111 or bit.band(sourceFlags, 0x511) == 0x511 then -- Checking source
		if event and string.find(event, "_DAMAGE") then
			-- Get recount-like feature
			if InCombatLockdown() then
				cbtDmgAmount = cbtDmgAmount or 0
				cbtStart = cbtStart or GetTime() - 3.5
			else
				cbtDmgAmount = 0
				cbtStart = GetTime() - 3.5
			end
			
			-- DPS crap
			if not dmghealAmount then -- Checking dps type
				cbtDmgAmount = cbtDmgAmount + meleeAmount -- Melee swing
			else
				cbtDmgAmount = cbtDmgAmount + dmghealAmount -- Anything else
			end
		end
		
		-- Heal crap
		if event and string.find(event, "_HEAL") then
			if InCombatLockdown() then
				cbtHealAmount = cbtHealAmount or 0
				cbtStart = cbtStart or GetTime() - 3.5
			else
				cbtHealAmount = 0
				cbtStart = GetTime() - 3.5
			end
			
			cbtHealAmount = cbtHealAmount + ( dmghealAmount - overHeal ) -- Only effective healing
		end
		
		-- Calc
		if cbtStart then
			self:Calculate()
		end
		
		-- Show
		if InCombatLockdown() then
			self:RefreshFrame()
		end
	end
end

function ShowMyDPS:Calculate()
	-- Calculation
	cbtDuration = GetTime() - cbtStart
	if cbtDmgAmount then 
		cbtDPS = (floor((cbtDmgAmount/cbtDuration * 100) + 0.5))/100
		if cbtDmgAmount > 1000000 then
			cbtDmgShown = floor(cbtDmgAmount/100000)/10 .."m"
		elseif cbtDmgAmount > 1000 then
			cbtDmgShown = floor(cbtDmgAmount/100)/10 .."k"
		else
			cbtDmgShown = cbtDmgAmount
		end
	end
	
	if cbtHealAmount then 
		cbtHPS = (floor((cbtHealAmount/cbtDuration * 100) + 0.5))/100
		if cbtHealAmount > 1000000 then
			cbtHealShown = floor(cbtHealAmount/100000)/10 .."m"
		elseif cbtHealAmount > 1000 then
			cbtHealShown = floor(cbtHealAmount/100)/10 .."k"
		else
			cbtHealShown = cbtHealAmount
		end
		
	end
end

function ShowMyDPS:RefreshFrame()
	if modes[self.db.char.mode] == "DPS" then
		if cbtDmgAmount and cbtDmgAmount > 0 then
			self.Frame.text:SetText(string.gsub(string.gsub(string.gsub(self.db.char.customDisplay, "$total", cbtDmgShown), "$persec", cbtDPS), "$time", SecondsToTime(cbtDuration)))
		else
			ShowMyDPSFrameText:SetText("Damage")
		end
	elseif modes[self.db.char.mode] == "HPS" then
		if cbtHealAmount and cbtHealAmount > 0 then
			self.Frame.text:SetText(string.gsub(string.gsub(string.gsub(self.db.char.customDisplay, "$total", cbtHealShown), "$persec", cbtHPS), "$time", SecondsToTime(cbtDuration)))
		else
			ShowMyDPSFrameText:SetText("Heal")
		end
	-- elseif modes[self.db.char.mode] == "DTPS" then
		-- ShowMyDPSFrameText:SetText("Damage taken")
	-- elseif modes[self.db.char.mode] == "HTPS" then
		-- ShowMyDPSFrameText:SetText("Heal recieved")
	end
end

function ShowMyDPS:LeaveCombat()
	self:RefreshFrame()
	if self.db.char.oocHide then self.Frame:Hide() end
	cbtDmgAmount = nil
	cbtHealAmount = nil
	cbtStart = nil
end